#===============================================================================
# Program:      discover.pl
#
# Description:  This script polls all devices on the specified subnet, creates
#               a list of SNMP or WMI manageable devices, and stores them in
#               discover.log.
#
#               This script is intended to be run by network administrators
#               against their own networks, where the SNMP read community string
#               and SNMP port are known.  It is of limited use otherwise.
#
#               In addition, as with all WMI-based scripts, the username under
#               which the script is run must have the rights to access WMI on
#               the remote hosts. This is usually the domain administrator
#               account.
#
#               The timeout values are set to the lowest possible number for
#               Net::Ping and Net::SNMP, the assumption being that the polled
#               hosts are on the local network. The script takes a little less
#               than 1 second for each polled host to run.
#
#               This script was developed under ActiveState Perl version 5.6.1.
#
# Requirements: ActiveState PERL must be installed on the system where the
#               script is executed.
#
#               The following additional Perl modules must also be installed:
#                 Net::Ping     - for testing device response with icmp ping
#                 Net::SNMP     - for all SNMP calls
#                 Win32::OLE    - for all WMI calls
#                 DBI           - for interfacing with MySQL
#                 DBD:MySQL     - for interfacing with MySQL
#
#               The following files must exist in the same directory as this
#               script:
#                 miscsubs.pl
#
# Known limitations:
#               1. Windows 2000/XP - none.
#               2. Windows NT4 - Not tested, but it should work after
#                   installing WMINT4.EXE (WBEMCORE v1.5) or later.
#               3. Windows 9x/ME - not tested. (Why waste your time?)
#               4. Non-Windows systems (Linux) - This script will NOT run.
#                   WMI calls can't be made from Linux (yet).
#
# Date       Author/Email                Description
# ---------- --------------------------- ---------------------------------------
# 10/27/2002 Jim Lancaster               Version 0.1 - Initial version.
#            jlancaster@mbdworld.com
#
# 02/27/2003 Jim Lancaster               Version 0.2
#            jlancaster@mbdworld.com     Increased speed by adding timeout
#                                         values to ping and snmp session tests.
#                                        Added option to write output to MySQL.
#                                        Moved common subroutines to miscsubs.pl
#
# 03/17/2003 Jim Lancaster               Version 0.2a
#                                        Streamlined the code. Minor fixes.
#                                        Referenced new subs_ scripts.
#
# 03/19/2003 Jim Lancaster               Version 0.2b
#                                        Fixed problem with RMON detection.
#                                        Changed command line options and
#                                         rewrote validation().
#
# 03/22/2003 Jim Lancaster               Version 0.2c
#                                        Fixed problem "-w" parameter.
# Known issues:
#   1. SNMP port appears to be open on every pingable device--even on a few
#       Windows servers where SNMP is not enabled. Apparently, I need to learn
#       more about sockets to make this work.
#
# Future plans:
#   1. None
#
#===============================================================================
use strict;

# Libraries of common subroutines. Start by looking in the current directory.
if (open(SUBS,"subs_misc.pl")) {
    require "subs_misc.pl";
    require "subs_snmp.pl";
    require "subs_cacti.pl";
# If this script is being called from within cacti, they will be found here.
} elsif (open(SUBS,"scripts/subs_misc.pl")) {
    require "scripts/subs_misc.pl";
    require "scripts/subs_snmp.pl";
    require "scripts/subs_cacti.pl";
} else {
    die "\n\nUnable to open subroutine libraries!";
}

# Used for command line parameters.
use Getopt::Long;

# Used to search for devices that respond to icmp ping.
use Net::Ping;
use vars qw($p);

# Used to test if SNMP port is open.
#use IO::Socket;
use Socket;

# Used to gather SNMP information from SNMP-enabled devices.
use Net::SNMP;
use vars qw($session $error $response);

# Used to gather WMI information from Windows devices.
use Win32::OLE qw(in);

# Used for writing results to cacti database.
use DBI;

#===============================================================================
#                              Constants
#===============================================================================
my $TRUE  = 1;
my $FALSE = 0;
my $YES = $TRUE;
my $NO = $FALSE;
my $SUCCESS = $TRUE;
my $FAILURE = $FALSE;
my $PROCESS_ID = $$;
my $SCRIPT_NAME = $0;
my $MAX_WAIT_TIME = 300;
my $WAIT_TIME = 10;
my $AUTHOR = "Jim Lancaster";
my $EMAIL = "jlancaster\@mbdworld.com";
my $COPYRIGHT = "Copyright 2003";
my $TITLE = "SNMP/WMI Discovery Tool";
my $VERSION = "Version 0.2b";
my $SCRIPT_OWNER = "MBD Network Services, LLC";
my $RETRIES=1;

#===============================================================================
#                              Script Parameters
#===============================================================================
# Create usage text.
my $usageText = "
$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER
$AUTHOR \($EMAIL\)

Usage: [perl] $SCRIPT_NAME -i=ipaddress or network/mask [-r -p -c -h]

 -i=ipaddress IP address or subnet to scan          (required)
 -r=string    SNMP read community string            (optional;default=public)
 -p=port      SNMP port number                      (optional;default=161)
 -c           Store output in discover_hosts table  (optional)
               in cacti.
 -w           Do not use WMI to determine host type (optional)
 -h,-?        Display this message                  (optional)

$SCRIPT_NAME polls either the ip address or all hosts on the specified
subnet, creates a list of SNMP or WMI manageable hosts and by default,
and stores the results in discover.log.

The -i parameter can be either a single IP address or a subnet indicated
by a trialing slash and the number of bits in the subnet mask.

Examples:
    perl $SCRIPT_NAME -i=192.168.1.21
    perl $SCRIPT_NAME -i=192.168.1.0/26 -r=\"xyz\" -p=162
    perl $SCRIPT_NAME -i=192.168.1.32/27 -c

The -r and -p parameters allow for specifying something other than the
default snmp read community string and snmp port.

If the -c parameter is used, the output will be inserted into the
discover_hosts table in cacti. (This requires some setup first. See the
readme.txt for further instructions.)

The -w parameter may be used on networks with no Windows servers to speed
up the discovery process by bypassing the use of WMI.

-----------------------------------------------------------------------------
Your feedback is strongly encouraged. This script should be flexible enough
to be stretched in a number of ways. All I need is your feedback to guide me.";

#===============================================================================
#                          Global Variables
#===============================================================================
# Initialize the device hash with default values.
my %device;
$device{'ipaddress'}                                = "unknown";
$device{'name'}                                     = "unknown";
$device{'type'}                                     = "unmanaged";
$device{'snmp'}                                     = "nosnmp";
$device{'community'}                                = "unknown";
$device{'port'}                                     = "unknown";
$device{'wmi'}                                      = "nowmi";
$device{'script'}                                   = "unknown";

my ($etherStatsEntryBaseOID,$etherStatsEntryOIDs)   = &mib2EtherStats();
my ($systemBaseOID,$systemOIDs)                     = &mib2System();

#===============================================================================
#                              Main Begin
#===============================================================================
# Read command line options.
my %parm = &cmdLineOpts();

# Go to usage if the command line input specifies it.
if ($parm{'help'}) {
    &usage($usageText)
}

my ($classC,$startip,$stopip) = &validate();

# If the read community string and port are not specified at the
#  command line, set them to the default values.
if (!$parm{'rcs'})  {$parm{'rcs'}="public"};
if (!$parm{'port'}) {$parm{'port'}="161"};

# Update the default values.
$device{'community'}    = $parm{'rcs'},;
$device{'port'}         = $parm{'port'},;

# Echo basic script information to the screen during runtime.
print "$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER\n";
print "$AUTHOR \($EMAIL\)\n\n";

# Set the script start time, which will be used to measure performance.
my $starttime = (time);

# Initialize some counters.
my $numHosts = 0;
my $respondingHosts = 0;

# Start the device discovery.
my %HoH_devices;
while ($startip <= $stopip) {

    # Increment the counter.
    $numHosts++;

    # Set the ip address for the device; reset the default values.
    $device{'ipaddress'} = "$classC.$startip";
    $device{'name'}      = "unknown";
    $device{'type'}      = "unmanaged";
    $device{'snmp'}      = "nosnmp";
    $device{'wmi'}       = "nowmi";
    $device{'script'}    = "unknown";

    print "$device{'ipaddress'}\t";

    # Ping the device using icmp.
    $p = Net::Ping->new('icmp');
    if ($p->ping($device{'ipaddress'},1)) {  #Timeout = 1 second.

        # Attempt to determine the device type.
        &deviceType();

        # Store the device information results in a hash of hashes.
        if ($device{'type'} ne "unmanaged") {
            $HoH_devices{$device{'ipaddress'}} = {%device};
        }

        # Increment the counter.
        $respondingHosts++;

    } else {
        print "\t\tNo response.\n";
    }

    # Close the ping session.
    $p->close();

    # Increment the last octet.
    $startip++;
}

if ($parm{'cacti'}) {

    # Get the cacti MySQL database settings;
    my %cactiDB = &cactiDB();
    my ($dsn)       = "DBI:mysql:$cactiDB{'default'}:$cactiDB{'hostname'}";
    my ($username)  = "$cactiDB{'username'}";
    my ($password)  = "$cactiDB{'password'}";

    my ($query,$dbh,$sth);

    # Connect to the cacti MySQL database.
    $dbh = DBI->connect($dsn,$username,$password)
        or die "Can't connect to MySQL database: $DBI::errstr\n";

    # First, clear the existing data from discover_hosts.
    $query = "DELETE from discover_hosts";
    $dbh->do($query)
        or die "\n\nUnable to execute query:\n$query\n$DBI::errstr";

    # Next, insert the newly disovered hosts into the table.
    $query = "INSERT INTO discover_hosts (ipaddress,name,type,snmp,
        snmp_community,snmp_port,wmi,script) VALUES (?,?,?,?,?,?,?,?)";
    $sth = $dbh->prepare ($query)
        or die "\n\nUnable to prepare query:\n$query\n$DBI::errstr";
    foreach my $key (keys %HoH_devices) {
        $sth->execute (
            $HoH_devices{$key}{'ipaddress'},
            $HoH_devices{$key}{'name'},
            $HoH_devices{$key}{'type'},
            $HoH_devices{$key}{'snmp'},
            $HoH_devices{$key}{'community'},
            $HoH_devices{$key}{'port'},
            $HoH_devices{$key}{'wmi'},
            $HoH_devices{$key}{'script'})
                or die "\n\nUnable to execute statement handle\n$DBI::errstr";
    }

    # Close the statement & database handles.
    $sth->finish();
    $dbh->disconnect ()
        or die "Disconnection failed: $DBI::errstr\n";

} else {

    # Write results to the log file.
    open CFG, ">discover.log"
        or die "Can't create discover.log. ($!)\n";
    foreach my $key (keys %HoH_devices) {
        print CFG "$HoH_devices{$key}{'ipaddress'},";
        print CFG "$HoH_devices{$key}{'name'},";
        print CFG "$HoH_devices{$key}{'type'},";
        print CFG "$HoH_devices{$key}{'wmi'},";
        print CFG "$HoH_devices{$key}{'snmp'},";
        print CFG "$HoH_devices{$key}{'script'},";
        print CFG "$HoH_devices{$key}{'port'},";
        print CFG "$HoH_devices{$key}{'community'},";

        #Add the poll time to the device line.
        print CFG "$starttime\n";
    }

    close CFG;
}

# Calculate the runtime statistics.
my $endtime = (time);
my $runtime = $endtime - $starttime;

# Echo report completion to the screen.
print "Discovery complete.\n\n";
print "Results: $respondingHosts hosts of $numHosts responded in $runtime seconds.\n";

#===============================================================================
#                              Main End
#===============================================================================

#===============================================================================
#                              Subroutines
#===============================================================================
# cmdLineOpts()
#
# Description - Check command line options
#
# Inputs:
#  1. None
#
# Processing:
#  1. Read the command line options and store the results in a hash.
#
# Outputs:
#  1. %parm - A hash containing the command line options.
#
#===============================================================================
sub cmdLineOpts
{
    # Check for command line options. Store them in %parm hash if present.
    my %parm;
    Getopt::Long::Configure("prefix_pattern=(-|--\\/)");
    GetOptions(\%parm, qw(
            ipaddress|i=s
            rcs|r=s
            port|p=s
            cacti|c
            nowmi|w
        ));

    if (%parm) {
        return(%parm);
    } else {
        &usage($usageText);
    }
}

#===============================================================================
# validate()
#
# Description - Validate the ip address or network/mask combination.
#
# Inputs:
#  1. None
#
# Processing:
#  1.
#
# Outputs:
#  1. $classC
#  2. $startip
#  3. $stopip
#
#===============================================================================
sub validate
{
    # Validate the ip address (range).
    if ($parm{'ipaddress'} !~ /[0-9]+\.[0-9]+\.[0-9]+\.([0-9]+|[0-9]+\/[0-9]+)/) {
        die "\"$parm{'ipaddress'}\" is not a valid ip address or network/mask combination!\n";
    }

    # Parse the ip address into octets.
    my @octets = split/\./,$parm{'ipaddress'};
    my $classC = "$octets[0].$octets[1].$octets[2]";

    # Parse the last octet.
    my ($startip,$stopip,$subnet);

    # Check to see if there is a subnet mask.
    if ($octets[3] =~ /\//) {

        ($startip,$subnet) = split /\//,$octets[3];

        # Validate the network number.
        if ($octets[0] > 254 || $octets[1] > 254 || $octets[2] > 254 || $startip > 254) {
            die "Invalid network number: $octets[0].$octets[1].$octets[2].$startip\n";
        }

        # Validate the subnet mask.
        if ($subnet <= 24 || $subnet >= 31) {
            die "Subnet mask ($subnet) must be in the range [24-31]\n";
        }

        # Validate the network/subnet mask combination
        my $numhosts = 2 ** (32 - $subnet);
        my $total = $startip;
        while ($total < 256) {
            $total = $total + $numhosts;
        }
        if ($total != 256) {
            die "Invalid network/mask combination: $parm{'ipaddress'}\n";
        }

        $stopip = $startip + ($numhosts - 2);
        $startip++;

    } else {

        # Validate the ip address.
        if ($octets[0] > 254 || $octets[1] > 254 || $octets[2] > 254 || $octets[3] > 254) {
            die "Invalid ip address: $octets[0].$octets[1].$octets[2].$octets[3]\n";
        }

        $startip = $octets[3];
        $stopip = $octets[3];

    }

    # For testing purposes only.
    #print "$classC,$startip,$stopip\n";

    return ($classC,$startip,$stopip);
}

#===============================================================================
# deviceType()
#
# Descripton - Use SNMP and/or WMI to determine the device type.
#
# Inputs:
#  1. None
#
# Processing:
#  1. Check to see if the SNMP port responds, then attempt to use SNMP
#      to determine the device type. If it is a Windows server or the
#      device type cannot be determined using SNMP, use WMI to determine
#      the device type.
#
# Outputs:
#  1.
#
#===============================================================================
sub deviceType
{

    # Check to see if SNMP port is responding
    my $sock = IO::Socket::INET->new(Proto    => "udp",
                                     PeerAddr => $device{'ipaddress'},
                                     PeerPort => $device{'port'},
                                     Type     => "SOCK_DGRAM");

    # If the SNMP port is open, attempt to use SNMP to gather
    #  more information about the device.
    if ($sock) {

        # SNMP is enabled.
        $device{'snmp'} = "snmp";
        $device{'type'} = "unknown";

        # Attempt to create an SNMP session.
        ($session, $error) = Net::SNMP->session(
            -hostname    => $device{'ipaddress'},
            -community   => $device{'community'},
            -port        => $device{'port'},
            -timeout     => 1
            );

        # Retrieve a hash of system OID values
        my ($systemBaseOID,$systemOIDs) = &mib2System();
        my %sysValues = &snmpValue($parm{'ipaddress'},$session,"0",$systemOIDs);

        # If the  SNMP session was successful, use SNMP to gather
        #  more information about the device.
        if ($sysValues{'sysDescr'} || $sysValues{'sysName'}) {

            # Test to see if RMON is available.
            &rmonTest();
            print "$device{'snmp'},";

            #  Attempt to use sysName for device name.
            if ($sysValues{'sysName'} ne "") {
                $device{'name'} = $sysValues{'sysName'};
                $device{'name'} =~ s/ /_/g;
            } else {
                $device{'name'} = "blank";
            }

            # sysDescr is the most likely place to find the device type.
            my $sysDescr = $sysValues{'sysDescr'};

            # Search through sysDescr to try and determine the device type.
            if ($sysDescr =~ /netware/i) {
                $device{'type'}     = "nwserver";
                $device{'script'}   = "unknown";
                print "$device{'wmi'}\t";
                print "$device{'name'} is a NetWare server!\n";
            } elsif ($sysDescr =~ /windows/i) {
                $device{'type'}     = "windows";
                $device{'script'}   = "w2kperf";

                # This is a Windows device with SNMP enabled. It could be
                #  either a server or a workstation. Refine the type via WMI.
                if (!$parm{'nowmi'}){&wmiTest();}

            } elsif ($sysDescr =~ /switch/i) {
                $device{'type'}     = "switch";
                $device{'script'}   = "switch";
                print "$device{'wmi'}\t";
                print "$device{'name'} is a managed switch!\n";
            } elsif ($sysDescr =~ /jetdirect/i) {
                $device{'type'}     = "hpprinter";
                $device{'script'}   = "printer";
                print "$device{'wmi'}\t";
                print "$device{'name'} is an HP printer!\n";
            } elsif ($sysDescr =~ /xerox/i) {
                $device{'type'}     = "xeroxprinter";
                $device{'script'}   = "printer";
                print "$device{'wmi'}\t";
                print "$device{'name'} is a Xerox printer!\n";
            } elsif ($sysDescr =~ /printer/i) {
                $device{'type'}     = "printer";
                $device{'script'}   = "printer";
                print "$device{'wmi'}\t";
                print "$device{'name'} is a printer!\n";
            } elsif ($sysDescr =~ /apc web/i) {
                $device{'type'}     = "apcups";
                $device{'script'}   = "unknown";
                print "$device{'wmi'}\t";
                print "$device{'name'} is an APC UPS!\n";
            } else {
                $device{'type'}     = "unknown";
                $device{'script'}   = "unknown";
                &rfcTest(\%sysValues);
            }

        } else {
            $device{'type'}     = "unknown";
            $device{'script'}   = "unknown";
            $device{'snmp'}     = "nosnmp";
            print "$device{'snmp'},";

            # Use WMI to test it further.
            if (!$parm{'nowmi'}){&wmiTest();}

            print "$device{'name'} is not manageable via SNMP\n";
            print "\t...Check SNMP port ($device{'port'}) or read community string ($device{'community'})\n";
        }

        # Close the SNMP session.
        $session->close;

    } else {
        $device{'type'}     = "unknown";
        $device{'script'}   = "unknown";
        $device{'snmp'}     = "nosnmp";
        print "$device{'snmp'},";

        # Use WMI to test it further.
        if (!$parm{'nowmi'}){&wmiTest();}

        print "$device{'name'} is not manageable via SNMP\n";
    }

    close $sock;
    return;
}

#===============================================================================
# wmiTest()
#
# Description -
#
# Inputs:
#  1. None
#
# Processing:
#  1. Attempt to connect to Windows device using WMI. If successful, update
#      name and type.
#
# Outputs:
#  1. None
#
#===============================================================================
sub wmiTest
{

    # From the WMI documentation, the possible values for 'DomainRole':
    #   0  Standalone Workstation
    #   1  Member Workstation
    #   2  Standalone Server
    #   3  Member Server
    #   4  Backup Domain Controller
    #   5  Primary Domain Controller

    my $class = "WinMgmts:{impersonationLevel=impersonate}!//$device{'ipaddress'}";
    if (my $wmi = Win32::OLE->GetObject($class)) {
        foreach my $dev (in($wmi->InstancesOf("Win32_ComputerSystem"))) {
            $device{'name'}     = $dev->{'Name'};
            $device{'wmi'}      = "wmi";
             $device{'script'}   = "w2kperf";
            if ($dev->{'DomainRole'} =~ /[2-5]/) {
                $device{'type'} = "winserver";
                print "$device{'wmi'}\t";
                print "$device{'name'} is a Windows server!\n";
            } elsif ($dev->{'DomainRole'} =~ /[0-1]/){
                $device{'type'} = "winwks";
                print "$device{'wmi'}\t";
                print "$device{'name'} is a Windows workstation!\n";
            } else {
                $device{'type'} = "windows";
                print "$device{'wmi'}\t";
                print "Unable to determine $device{'name'} type using WMI.\n";
            }
        }

    } elsif ($device{'script'} eq "w2kperf") {

        # This is not a WMI manageable device.
        $device{'wmi'} = "nowmi";

        print "$device{'wmi'}\t";
        print "$device{'name'} is some type of Windows device.\n";
        print "\t...Is WMI installed? Do you have the rights to access it?\n";

    } else {

        # This is not a WMI manageable device.
        $device{'wmi'} = "nowmi";
        print "$device{'wmi'}\t";
    }

    return;
}

#===============================================================================
# rmonTest ()
#
# Description -
#
# Inputs:
#  1. None
#
# Processing:
#  1. Attempt to read the RMON etherStats tables.
#
# Outputs:
#  1. None
#
#===============================================================================
sub rmonTest
{
    # Retrieve hash of hashes of ifEntry OID values
    my ($etherStatsEntryBaseOID,$etherStatsEntryOIDs) = &mib2EtherStats();
    my %hoh_rmon = &snmpTable($device{'ipaddress'},$session,$$etherStatsEntryOIDs{'rmonIndex'},$etherStatsEntryOIDs);

    if (%hoh_rmon) {
        $device{'snmp'} = "rmon";
    }

    return;
 }

#===============================================================================
# rmonTest ($sysValues)
#
# Description -
#
# Inputs:
#  1. $sysValues - A reference to a hash containing MIB2 System values.
#
# Processing:
#  1.
#
# Outputs:
#  1. None
#
#===============================================================================
sub rfcTest
{

    my $sysValues = shift;

    # If we are unable to determine the device type by
    #  examining sysDescr, then use other means.
    # From RFC1213-MIB:
    #   1  physical (e.g., repeaters)
    #   2  datalink/subnetwork (e.g., bridges)
    #   3  internet (e.g., IP gateways)
    #   4  end-to-end  (e.g., IP hosts)
    #   7  applications (e.g., mail relays)
    my %rfc1213 = (
        1,"repeater",
        2,"switch",
        3,"router"
    );
    my $type = $rfc1213{$$sysValues{'sysServices'}};

    if ($type) {
        $device{'type'} = $type;
        print "$device{'name'} is a managed $device{'type'}!\n";
    } else {
        # Use WMI to test it further.
        if (!$parm{'nowmi'}){&wmiTest();}
        print "$device{'name'} is SNMP manageable, type $device{'type'}.\n";
    }

    return;
}

#===============================================================================
