#===============================================================================
# Program:      printer.pl
#
# Description:  This script gathers basic printer performance information via
#               SNMP for use with cacti v0.6.8a.  It is also capable of creating
#               the DIS, DS entries in cacti for the polled object, as well as
#               a set of basic graphs.
#
#               This script was developed under ActiveState Perl version 5.6.1.
#
# Requirements: This ActiveState PERL must be installed on the system where the
#               script is executed.
#
#               The following additional Perl modules must also be installed:
#                 Net::Ping     - for testing device response with icmp ping
#                 Net::SNMP     - for all SNMP calls
#                 DBI           - for interfacing with MySQL
#                 DBD:MySQL     - for interfacing with MySQL
#
#               The following files must exist in the same directory as this
#               script:
#                   subs_misc.pl - A library of miscellaneous subroutines.
#                   subs_snmp.pl - A library of SNMP subroutines.
#                   subs_cacti.pl - A library of cacti subroutines.
#
#               Known limitations:
#                   1. This script polls the mib2 printer OIDs and not the
#                       vendor enterprise mibs. Some of the older HP printers
#                       I tested (HP4SI) did not respond to the page count polls.
#
# Date       Author/Email                Description
# ---------- --------------------------- ---------------------------------------
# 03/11/2003 Jim Lancaster               Version 0.1
#            jlancaster@mbdworld.com
#
# 03/11/2003 Jim Lancaster               Version 0.1a
#            jlancaster@mbdworld.com     Corrected usage.
#
# 03/19/2003 Jim Lancaster               Version 0.1b
#            jlancaster@mbdworld.com     Corrected problem with @output if the
#                                         printer does not support MIB2 printer.
#
# Known issues:
#   1. None
#
# Future plans:
#   1. None
#
#===============================================================================
use strict;

# Libraries of common subroutines. Start by looking in the current directory.
if (open(SUBS,"subs_misc.pl")) {
    require "subs_misc.pl";
    require "subs_snmp.pl";
    require "subs_cacti.pl";
# If this script is being called from within cacti, they will be found here.
} elsif (open(SUBS,"scripts/subs_misc.pl")) {
    require "scripts/subs_misc.pl";
    require "scripts/subs_snmp.pl";
    require "scripts/subs_cacti.pl";
} else {
    die "\n\nUnable to open subroutine libraries!";
}

# Used for command line parameters.
use Getopt::Long;

# Used to search for devices that respond to icmp ping.
use Net::Ping;
use vars qw($p);

# Used to test if SNMP port is open.
use IO::Socket;

# Used to gather SNMP information from SNMP-enabled devices.
use Net::SNMP;
use vars qw($session $error $response);

# Used for writing results to cacti database.
use DBI;

#===============================================================================
#                   Constants
#===============================================================================
my $TRUE  = 1;
my $FALSE = 0;
my $YES = $TRUE;
my $NO = $FALSE;
my $SUCCESS = $TRUE;
my $FAILURE = $FALSE;
my $PROCESS_ID = $$;
my $SCRIPT_NAME = $0;
my $MAX_WAIT_TIME = 300;
my $WAIT_TIME = 10;
my $AUTHOR = "Jim Lancaster";
my $EMAIL = "jlancaster\@mbdworld.com";
my $COPYRIGHT = "Copyright 2003";
my $TITLE = "Printer Performance Report";
my $VERSION = "Version 0.1a";
my $SCRIPT_OWNER = "MBD Network Services, LLC";

#===============================================================================
#                   Script Parameters
#===============================================================================
my $usageText = "
$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER
$AUTHOR \($EMAIL\)


Usage: [perl] $SCRIPT_NAME -i=ipaddress [-q -h -? -c]

  -i=ipaddress IP address of printer                    (required)

  -q           Return data only--no headers or footers  (optional)
  -h,-?        Display usage text                       (optional)
  -c           Create the cacti DIS, DS & graphs        (optional)

$SCRIPT_NAME retrieves printer performance information using SNMP.

Without the '-q' parameter, the script prints the the gathered
information to the screen. Use this for testing purposes to see if the
remote device is accessible and to see the number of host attributes
the script is returning performance data for.

With the '-q' parameter, the script returns the performance data in a
single line format suitable for use with Cacti.

With the '-c' parameter, this script will create the DIS (if necessary),
the DS, and a basic set of graphs. This should allow you to begin
graphing data within minutes of successfully installing cacti. The graphs
can be modified to suit your needs using cacti's built-in tools at any
time.

Your feedback is welcome. This script should be flexible enough to be
stretched in a number of ways. All I need is your input to guide me.";

#===============================================================================
#                   Reports Array
#===============================================================================

#===============================================================================
#                   Global Variables
#===============================================================================
# From Main
my (%sysValues,%hoh_if,%hoh_prn);
my (@output,@outputStr);

#===============================================================================
#                   Main Begin
#===============================================================================
# Read command line options.
my %parm = &cmdLineOpts();

# Echo basic script information to the screen during runtime.
if (!$parm{'quiet'}) {
    print STDOUT "$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER\n";
    print STDOUT "$AUTHOR \($EMAIL\)\n\n";
}

# Go to usage if the command line input specifies it.
if ($parm{'help'}) {
    &usage($usageText)
};

# Go to usage if no IP address is specified.
if (!$parm{'ipaddress'}) {
    &usage($usageText)
};

# If the read community string and port are not specified at the
#  command line, set them to the default values.
if (!$parm{'rcs'}) {$parm{'rcs'}="public"};
if (!$parm{'port'}) {$parm{'port'}="161"};

# Ping the device using icmp.
$p = Net::Ping->new('icmp');
if ($p->ping($parm{'ipaddress'},1)) {  # Timeout = 1 second.

    # Get the SNMP performance data.
    my $hostname = &snmpStats();

    if ($hostname) {

        # Display contents of hashes.
        if (!$parm{'quiet'}) {
            print "Print results for $parm{'ipaddress'}:\n\n";
            print "Printer name => $hostname\n";
            &dref(\%hoh_prn,'hoh');
            &dref(\%hoh_if,'hoh');
        }

        # Build output & output string.
        &buildOutput();

        # Echo report completion to the screen, or print data in Cacti acceptable format.
        if (!$parm{'quiet'}) {
            print STDOUT "\nResults for $parm{'ipaddress'} complete.\n\n"
        } else {
            # Print output in Cacti acceptable format:
            print join(" ",@output);
        }

        # Configure Cacti to poll and graph this host if the command line input specifies it.
        if ($parm{'create'}) {

            # Get the cacti MySQL database settings;
            my %cactiDB     = &cactiDB();
            my ($dsn)       = "DBI:mysql:$cactiDB{'default'}:$cactiDB{'hostname'}";
            my ($username)  = "$cactiDB{'username'}";
            my ($password)  = "$cactiDB{'password'}";

            # Connect to the cacti MySQL database.
            my $dbh = DBI->connect($dsn,$username,$password)
                or die "Can't connect to MySQL database: $DBI::errstr\n";

            # Link the parent record to the standard rra's. There are 4 of them.
            my $query = "SELECT name FROM cacti.discover_hosts WHERE ipaddress = '$parm{'ipaddress'}'";
            my $newHostname = $dbh->selectrow_array ($query);
            if ($newHostname) {
                $hostname = $newHostname;
            }

            # For testing purposes only.
            print "prn count = ".values(%hoh_prn)."\tif count = ".values(%hoh_if)."\n";

            my $countPrn    = values(%hoh_prn);
            my $countIf     = values(%hoh_if);
            my $DISName     = "Get Printer Performance via SNMP (P-$countPrn/I-$countIf)";

            # Create the data input source (DIS)
            my ($newSrcID,$inFieldID,$outFieldID)  = &createDIS($dbh,$DISName,$parm{'ipaddress'},'printer.pl -i=<ipaddress> -q',\@outputStr);

            # Create the srcInput hash. There should be one for each input field specified
            #  in '<>' in the preceding command.
            my %srcInput;
            $srcInput{'ipaddress'}  = $parm{'ipaddress'};

            # In order to keep track of the type of hash that is referenced in srcData array,
            #  each array element number needs to be predetermined. Unused elements can be
            #  commented-out.
            my @srcData;
            #$srcData[0]     = \%hoh_cpu;
            #$srcData[1]     = \%hoh_mem;
            #$srcData[2]     = \%hoh_disk;
            $srcData[3]     = \%hoh_if;
            $srcData[4]     = \%hoh_prn;

            # For testing purposes only.
            #dref(\@srcData,"array");

            # Create the data source (DS)
            my ($ds,$newDSID)   = &createDS($dbh,$hostname,$newSrcID,$inFieldID,$outFieldID,\%srcInput,\@srcData);

            # Create the graphs
            my ($rrdGraphID)    = &createGraph($dbh,$hostname,$newDSID,\@srcData);

            # Add the graphs to the graph hierarchy.
            my %hierarchy;
            $hierarchy{'root'}       = "Auto-created";
            $hierarchy{'heading'}    = "Printers";
            #$hierarchy{'hostname'}   = $hostname;                  # Optional.
            $hierarchy{'device'}     = $hostname;
            &createGraphHierarchyTree($dbh,$rrdGraphID,\%hierarchy);

            # Terminate the database connection.
            $dbh->disconnect ()
                or die "Disconnection failed: $DBI::errstr\n";
        }
    }

} else {

    print "$parm{'ipaddress'} - did not respond.\n";

}

#===============================================================================
#                   Main End
#===============================================================================

#===============================================================================
#                   Subroutines
#===============================================================================
# cmdLineOpts()
#
# Description - Check command line options
#
# Inputs:
#  1. None
#
# Processing:
#  1. Read the command line options and store the results in a hash.
#
# Outputs:
#  1. %parm - A hash containing the command line options.
#
#===============================================================================
sub cmdLineOpts
{
    # Check for command line options. Store them in %parm hash if present.
    my %parm;
    Getopt::Long::Configure("prefix_pattern=(-|--\\/)");
    GetOptions(\%parm, qw(
                help|?|h
                ipaddress|i=s
                rcs|r=s
                port|p=s
                create|c
                quiet|q
                ));

    return(%parm);
}

#===============================================================================
# snmpStats()
#
# Descripton - Use SNMP to gather the performance data.
#
# Inputs:
#  1.
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#===============================================================================
sub snmpStats
{

    # Check to see if SNMP port is responding
    my $remote = IO::Socket::INET -> new(
                    Proto      => "udp",
                    PeerAddr   => $parm{'ipaddress'},
                    PeerPort   => $parm{'port'},
                    );

    # If the SNMP port is open, attempt to use SNMP to gather
    #  more information about the device.
    if ($remote) {
        close $remote;

        # Attempt to create an SNMP session.
        ($session, $error) = Net::SNMP->session(
            -hostname    => $parm{'ipaddress'},
            -community   => $parm{'rcs'},
            -port        => $parm{'port'},
            -timeout     => 1
            );

        # Retrieve a hash of system OID values
        my ($systemBaseOID,$systemOIDs) = &mib2System();
        %sysValues = &snmpValue($parm{'ipaddress'},$session,"0",$systemOIDs);

        # Fix those conditions where sysName has not been set.
        if (defined($sysValues{'sysDescr'}) & $sysValues{'sysName'} eq "") {
            $sysValues{'sysName'} = "blank";
        }

        # Retrieve hash of hashes of ifEntry OID values
        my ($ifEntryBaseOID,$ifEntryOIDs) = &mib2If();
        my %ifValues = &snmpTable($parm{'ipaddress'},$session,$$ifEntryOIDs{'ifIndex'},$ifEntryOIDs);

        # Store the interesting network traffic in the hoh_if hash of hashes.
        foreach my $key1 (keys %ifValues) {
            foreach my $key2 (keys %{$ifValues{$key1}}) {
                if (${$ifValues{$key1}}{'ifType'} =~ /(6|53)/ & $key2 =~ /(Octets|Errors|Discards|Descr)$/) {
                    $hoh_if{$key1}{$key2} = $ifValues{$key1}{$key2};
                }
            }
        }

        # Retrieve hash of prtMarkerEntry OID values.
        # For some reason, the instance in my tests always showed up as "1.1".  Normally, I try to
        #  find an index OID and do a snmpTable() to get the instances (see the previous ifEntry
        #  code), but that didn't work for the HP LJ8000 and Xerox Phaser 6200 printers I tested.
        my ($prtMarkerEntryBaseOID,$prtMarkerEntryOIDs) = &mib2prt();
        my %prtValues = &snmpValue($parm{'ipaddress'},$session,"1.1",$prtMarkerEntryOIDs);

        # Store the results in a hash of hashes.
        #$hoh_prn{'Printer'} = {%prtValues};
        if (%prtValues) {
            $hoh_prn{'Printer'} = {%prtValues};
        }

        $session->close;
    }

    # For testing purposes only.
    #dref(\%hoh_if,"hoh");
    #dref(\%hoh_prn,"hoh");

    return($sysValues{'sysName'});
}

#===============================================================================
# buildOutput()
#
# Description - Build output & outputStr for Cacti.
#
# Inputs:
#  1.
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#===============================================================================
sub buildOutput
{

    # Printer page counter
    if (%hoh_prn) {
        push @output,$hoh_prn{'Printer'}{'prtMarkerLifeCount'};
        push @outputStr,"prtMarkerLifeCount";
    }

    # Interface
    foreach my $key (sort keys %hoh_if) {
        push @output,$hoh_if{$key}{'ifInOctets'};
        push @output,$hoh_if{$key}{'ifInErrors'};
        push @output,$hoh_if{$key}{'ifInDiscards'};
        push @output,$hoh_if{$key}{'ifOutOctets'};
        push @output,$hoh_if{$key}{'ifOutErrors'};
        push @output,$hoh_if{$key}{'ifOutDiscards'};
        push @outputStr,"ifInOctets-".$key;
        push @outputStr,"ifInErrors-".$key;
        push @outputStr,"ifInDiscards-".$key;
        push @outputStr,"ifOutOctets-".$key;
        push @outputStr,"ifOutErrors-".$key;
        push @outputStr,"ifOutDiscards-".$key;
    }

    # For testing purposes only.
    #print join("\n",@outputStr)."\n";
    #print join(",",@output)."\n";

}

#===============================================================================
