#===============================================================================
# Program:      rollback.pl
#
# Description:  This script determines the last record id in use in the cacti
#               database tables and creates 'rollback.sql' MySQL query to
#               put the cacti tables back in the current state.
#
#               This script was developed under ActiveState Perl version 5.6.1.
#
# Requirements: This ActiveState PERL must be installed on the system where the
#               script is executed.
#
#               The following additional Perl modules must also be installed:
#                   DBI           - for interfacing with MySQL
#                   DBD:MySQL     - for interfacing with MySQL
#
#               The following files must exist in the same directory as this
#               script:
#                   subs_misc.pl - A library of miscellaneous subroutines.
#                   subs_cacti.pl - A library of cacti subroutines.
#
#               Known limitations:
#                   1. None
#
# Date       Author/Email                Description
# ---------- --------------------------- ---------------------------------------
# 03/18/2003 Jim Lancaster               Version 0.1
#            jlancaster@mbdworld.com
#
# 03/21/2003 Jim Lancaster               Version 0.1a
#                                        Fixed problem with graph_hierarch_items
#
#
# Known issues:
#   1. None
#
# Future plans:
#   1. None
#
#===============================================================================
use strict;

# Libraries of common subroutines. Start by looking in the current directory.
if (open(SUBS,"subs_cacti.pl")) {
    #require "subs_misc.pl";
    require "subs_cacti.pl";
# If this script is being called from within cacti, they will be found here.
} elsif (open(SUBS,"scripts/subs_cacti.pl")) {
    #require "scripts/subs_misc.pl";
    require "scripts/subs_cacti.pl";
} else {
    die "\n\nUnable to open subroutine libraries!";
}

# Used for command line parameters.
use Getopt::Long;

# Used for writing results to cacti database.
use DBI;

#===============================================================================
#           Constants
#===============================================================================
my $TRUE  = 1;
my $FALSE = 0;
my $YES = $TRUE;
my $NO = $FALSE;
my $SUCCESS = $TRUE;
my $FAILURE = $FALSE;
my $PROCESS_ID = $$;
my $SCRIPT_NAME = $0;
my $MAX_WAIT_TIME = 300;
my $WAIT_TIME = 10;
my $AUTHOR = "Jim Lancaster";
my $EMAIL = "jlancaster\@mbdworld.com";
my $COPYRIGHT = "Copyright 2003";
my $TITLE = "Create cacti 'rollback' MySQL Query";
my $VERSION = "Version 0.1";
my $SCRIPT_OWNER = "MBD Network Services, LLC";

#===============================================================================
#           Script Parameters
#===============================================================================
my $usageText = "
$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER
$AUTHOR \($EMAIL\)


Usage: [perl] $SCRIPT_NAME [-h -?]

  -h,-?        Display usage text                       (optional)

$SCRIPT_NAME This script determines the last record id in use in the cacti
database tables and creates 'rollback.sql' MySQL query to put the cacti tables
back in the current state.

Your feedback is welcome. This script should be flexible enough to be
stretched in a number of ways. All I need is your input to guide me.";

#===============================================================================
#           Reports Array
#===============================================================================

#===============================================================================
#           Global Variables
#===============================================================================

#===============================================================================
#           Main Begin
#===============================================================================

# Read command line options.
my %parm = &cmdLineOpts();

# Echo basic script information to the screen during runtime.
print STDOUT "$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER\n";
print STDOUT "$AUTHOR \($EMAIL\)\n\n";

# Go to usage if the command line input specifies it.
if ($parm{'help'}) {
    &usage($usageText)
};

# Get the cacti MySQL database settings;
my %cactiDB     = &cactiDB();
my ($dsn)       = "DBI:mysql:$cactiDB{'default'}:$cactiDB{'hostname'}";
my ($username)  = "$cactiDB{'username'}";
my ($password)  = "$cactiDB{'password'}";

# Connect to the cacti MySQL database.
my $dbh = DBI->connect($dsn,$username,$password)
    or die "Can't connect to MySQL database: $DBI::errstr\n";

# Link the parent record to the standard rra's. There are 4 of them.
my %table;
${$table{'src'}}{'key'} = "ID";
${$table{'src_fields'}}{'key'} = "SrcID";
${$table{'src_data'}}{'key'} = "FieldID";
${$table{'rrd_ds'}}{'key'} = "ID";
${$table{'lnk_ds_rra'}}{'key'} = "DSID";
${$table{'rrd_graph'}}{'key'} = "ID";
${$table{'rrd_graph_item'}}{'key'} = "GraphID";
${$table{'graph_hierarchy'}}{'key'} = "ID";
${$table{'graph_hierarchy_items'}}{'key'} = "ID";

foreach my $key (keys %table) {
    ${$table{$key}}{'max'} = $dbh->selectrow_array ("select max(${$table{$key}}{'key'}) from $key");
}

# Terminate the database connection.
$dbh->disconnect ()
    or die "Disconnection failed: $DBI::errstr\n";

# Write results to the cfg file.
open SQL,">rollback.sql"
    or die "Can't create rollback.sql $!\n";

foreach my $key (keys %table) {
    print SQL "delete from $key where ${$table{$key}}{'key'} > ${$table{$key}}{'max'}\;\n";
}

close SQL;

#===============================================================================
#           Main End
#===============================================================================

#===============================================================================
#           Subroutines
#===============================================================================
# cmdLineOpts()
#
# Description - Check command line options
#
# Inputs:
#  1. None
#
# Processing:
#  1. Read the command line options and store the results in a hash.
#
# Outputs:
#  1. %parm - A hash containing the command line options.
#
#===============================================================================
sub cmdLineOpts
{
    # Check for command line options. Store them in %parm hash if present.
    my %parm;
    Getopt::Long::Configure("prefix_pattern=(-|--|\/)");
    GetOptions(\%parm, qw(
        help|?|h
        ));

    return(%parm);
}

#===============================================================================
