#=============================================================================
# Program:     subs_snmp.pl
#
# Description: Library of SNMP subroutines and MIB hashes.
#              For those OIDs that can be retrieved as a table, there is a
#              defined base OID. An "instance" is also defined where possible.
#
#               This script was developed under ActiveState Perl version 5.6.1.
#
# Requirements: This ActiveState PERL must be installed on the system where the
#               script is executed.
#
#               The following additional Perl modules must also be installed:
#                   None
#
#               The following files must exist in the same directory as this
#               script:
#                   None
#
#               Known limitations:
#                   None
#
#
# Date       Author/Email                Description
# ---------- --------------------------- -------------------------------------
# 03/17/2003 Jim Lancaster               Version 0.1 - Initial version
#            jlancaster@mbdworld.com
#
#
# Known issues:
#  1. None
#
# Future plans:
#  1. Add other mibs as needed.
#
#=============================================================================
use strict;
return 1;

#=============================================================================
#                           Subroutines
#=============================================================================
# snmpValue($hostname,$session,$instance,$hrOIDs)
#
# Description: Poll a device for SNMP values.
#
# Inputs:
#  1. $hostname - The hostname or ip address of the target
#  2. $session  - The SNMP session
#  3. $instance - The SNMP instance of the OIDs in %{$hrOIDs}.
#  4. $hrOIDs   - A reference to a hash of SNMP OIDs
#
# Processing:
#  1. Poll the SNMP device and store the results in a hash.
#
# Outputs:
#  1. %results - A hash of SNMP values
#
#=============================================================================
sub snmpValue
{

    my ($hostname,$session,$instance,$hrOIDs) = @_;

    # Initialize local variables.
    my (%results,@oids);

    # Set the basic system MIB OIDs.
    foreach my $key (keys %{$hrOIDs}) {
        push @oids,${$hrOIDs}{$key}.".".$instance;
    }

    # Poll the device.
    if (defined (my $response = $session->get_request(@oids))) {

        # If it returns anything, store the results in a hash.
        foreach my $key (keys %{$hrOIDs}) {
            $results{$key} = $response->{${$hrOIDs}{$key}.".".$instance};
        }
    }

    return (%results);
}

#=============================================================================
# snmpTable ($hostname,$session,$baseOID,$hrOIDs)
#
# Description: Poll a device for a table of SNMP values (instances). Use these
#               instances to poll for snmpValues.
#
# Inputs:
#  1. $hostname - The hostname or ip address of the target
#  2. $session  - The SNMP session
#  3. $baseOID  - The index OID in $hrOIDs.
#  4. $hrOIDs   - A reference to a hash of SNMP OIDs
#
# Processing:
#  1.
#
# Outputs:
#  1. %results  - A hash of SNMP value hashes.
#
#=============================================================================
sub snmpTable
{

    # Initialize the variables.
    my ($hostname,$session,$baseOID,$hrOIDs) = @_;
    my %results;

    # Retrieve a table using the base OID.
    if (defined (my $response = $session->get_table($baseOID))) {
        foreach my $key1 (sort keys %$response) {

            # Create an annonymous reference to the hash of SNMP values.
            $results{$$response{$key1}} = {&snmpValue($hostname,$session,$$response{$key1},$hrOIDs)};
        }
    }

    return (%results);

}

#=============================================================================
# mib2System()
#
# Description: Define SNMP MIB2 OIDs.
#
# Inputs:
#  1. None
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#=============================================================================
sub mib2System
{

    # mib-2.system
    #  Instance = "0"
    my $systemBaseOID                       = '1.3.6.1.2.1.1';
    my %systemOIDs;
    $systemOIDs{'sysDescr'}                 = $systemBaseOID.'.1';
    $systemOIDs{'sysObjectID'}              = $systemBaseOID.'.2';
    $systemOIDs{'sysUpTime'}                = $systemBaseOID.'.3';
    $systemOIDs{'sysContact'}               = $systemBaseOID.'.4';
    $systemOIDs{'sysName'}                  = $systemBaseOID.'.5';
    $systemOIDs{'sysLocation'}              = $systemBaseOID.'.6';
    $systemOIDs{'sysServices'}              = $systemBaseOID.'.7';

    return ($systemBaseOID,\%systemOIDs);
}

#=============================================================================
# mib2If()
#
# Description: Define SNMP MIB2 OIDs.
#
# Inputs:
#  1. None
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#=============================================================================
sub mib2If
{
    # mib-2.interfaces.ifTable.ifEntry
    #  Instance = "ifIndex"
    my $ifEntryBaseOID                      = '1.3.6.1.2.1.2.2.1';
    my %ifEntryOIDs;
    $ifEntryOIDs{'ifIndex'}                 = $ifEntryBaseOID.'.1';
    $ifEntryOIDs{'ifDescr'}                 = $ifEntryBaseOID.'.2';
    $ifEntryOIDs{'ifType'}                  = $ifEntryBaseOID.'.3';
#    $ifEntryOIDs{'ifMtu'}                  = $ifEntryBaseOID.'.4';
#    $ifEntryOIDs{'ifSpeed'}                = $ifEntryBaseOID.'.5';
#    $ifEntryOIDs{'ifPhysAddress'}          = $ifEntryBaseOID.'.6';
#   $ifEntryOIDs{'ifAdminStatus'}           = $ifEntryBaseOID.'.7';
    $ifEntryOIDs{'ifOperStatus'}            = $ifEntryBaseOID.'.8';
#    $ifEntryOIDs{'ifLastChange'}           = $ifEntryBaseOID.'.9';
    $ifEntryOIDs{'ifInOctets'}              = $ifEntryBaseOID.'.10';
#    $ifEntryOIDs{'ifInUcastPkts'}          = $ifEntryBaseOID.'.11';
#    $ifEntryOIDs{'ifInNUcastPkts'}         = $ifEntryBaseOID.'.12';
    $ifEntryOIDs{'ifInDiscards'}            = $ifEntryBaseOID.'.13';
    $ifEntryOIDs{'ifInErrors'}              = $ifEntryBaseOID.'.14';
#    $ifEntryOIDs{'ifInUnknownProtos'}      = $ifEntryBaseOID.'.15';
    $ifEntryOIDs{'ifOutOctets'}             = $ifEntryBaseOID.'.16';
#    $ifEntryOIDs{'ifOutUcastPkts'}         = $ifEntryBaseOID.'.17';
#    $ifEntryOIDs{'ifOutNUcastPkts'}        = $ifEntryBaseOID.'.18';
    $ifEntryOIDs{'ifOutDiscards'}           = $ifEntryBaseOID.'.19';
    $ifEntryOIDs{'ifOutErrors'}             = $ifEntryBaseOID.'.20';
#    $ifEntryOIDs{'ifOutQLen'}              = $ifEntryBaseOID.'.21';
#    $ifEntryOIDs{'ifSpecific'}             = $ifEntryBaseOID.'.22';

    return ($ifEntryBaseOID,\%ifEntryOIDs);
}

#=============================================================================
# mib2EtherStats()
#
# Description: Define SNMP MIB2 OIDs.
#
# Inputs:
#  1. None
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#=============================================================================
sub mib2EtherStats
{

    # mib-2.rmon.statistics.etherStatsTable.etherStatsEntry
    #  Instance = "etherStatsIndex" = "ifIndex"
    # Note: I had to change the OID descriptions to get them to fit under
    #  RRDtool's 19 character limit.
    my $etherStatsEntryBaseOID                             = '1.3.6.1.2.1.16.1.1.1';
    my %etherStatsEntryOIDs;
    $etherStatsEntryOIDs{'rmonIndex'}               = $etherStatsEntryBaseOID.'.1';
    $etherStatsEntryOIDs{'rmonDataSource'}          = $etherStatsEntryBaseOID.'.2';
    $etherStatsEntryOIDs{'rmonDropEvents'}          = $etherStatsEntryBaseOID.'.3';
    $etherStatsEntryOIDs{'rmonOctets'}              = $etherStatsEntryBaseOID.'.4';
    $etherStatsEntryOIDs{'rmonPkts'}                = $etherStatsEntryBaseOID.'.5';
    $etherStatsEntryOIDs{'rmonBroadcastPkts'}       = $etherStatsEntryBaseOID.'.6';
    $etherStatsEntryOIDs{'rmonMulticastPkts'}       = $etherStatsEntryBaseOID.'.7';
    $etherStatsEntryOIDs{'rmonCRCAlignErrors'}      = $etherStatsEntryBaseOID.'.8';
    $etherStatsEntryOIDs{'rmonUndersizePkts'}       = $etherStatsEntryBaseOID.'.9';
    $etherStatsEntryOIDs{'rmonOversizePkts'}        = $etherStatsEntryBaseOID.'.10';
    $etherStatsEntryOIDs{'rmonFragments'}           = $etherStatsEntryBaseOID.'.11';
    $etherStatsEntryOIDs{'rmonJabbers'}             = $etherStatsEntryBaseOID.'.12';
    $etherStatsEntryOIDs{'rmonCollisions'}          = $etherStatsEntryBaseOID.'.13';
    $etherStatsEntryOIDs{'rmonPkts64Octets'}        = $etherStatsEntryBaseOID.'.14';
    $etherStatsEntryOIDs{'rmonPkts65to127Octe'}     = $etherStatsEntryBaseOID.'.15';
    $etherStatsEntryOIDs{'rmonPkts128to255Oct'}     = $etherStatsEntryBaseOID.'.16';
    $etherStatsEntryOIDs{'rmonPkts256to511Oct'}     = $etherStatsEntryBaseOID.'.17';
    $etherStatsEntryOIDs{'rmonPkts512to1023Oc'}     = $etherStatsEntryBaseOID.'.18';
    $etherStatsEntryOIDs{'rmonPkts1024to1518O'}     = $etherStatsEntryBaseOID.'.19';
#    $etherStatsEntryOIDs{'rmonOwner'}               = $etherStatsEntryBaseOID.'.20';
#    $etherStatsEntryOIDs{'rmonStatus'}              = $etherStatsEntryBaseOID.'.21';

    return ($etherStatsEntryBaseOID,\%etherStatsEntryOIDs);

    return ($etherStatsEntryBaseOID,\%etherStatsEntryOIDs);
}

#=============================================================================
# mib2prt()
#
# Description: Define SNMP MIB2 OIDs.
#
# Inputs:
#  1. None
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#=============================================================================
sub mib2prt
{

    # mib-2.printmib.prtMarker.prtMarkerTable.prtMarkerEntry
    my $prtMarkerEntryBaseOID               = '1.3.6.1.2.1.43.10.2.1';
    my %prtMarkerEntryOIDs;
    $prtMarkerEntryOIDs{'prtMarkerLifeCount'} = $prtMarkerEntryBaseOID.'.4';

    return ($prtMarkerEntryBaseOID,\%prtMarkerEntryOIDs);
}

#=============================================================================
