#===============================================================================
# Program:      switch.pl
#
# Description:  This script gathers basic switch performance information via
#               SNMP for use with cacti v0.6.8a.  It is also capable of creating
#               the DIS, DS entries in cacti for the polled object, as well as
#               a set of basic graphs.
#
#               This script was developed under ActiveState Perl version 5.6.1.
#
# Requirements: This ActiveState PERL must be installed on the system where the
#               script is executed.
#
#               The following additional Perl modules must also be installed:
#                   Net::Ping     - for testing device response with icmp ping
#                   Net::SNMP     - for all SNMP calls
#                   DBI           - for interfacing with MySQL
#                   DBD:MySQL     - for interfacing with MySQL
#
#               The following files must exist in the same directory as this
#               script:
#                   subs_misc.pl - A library of miscellaneous subroutines.
#                   subs_snmp.pl - A library of SNMP subroutines.
#                   subs_cacti.pl - A library of cacti subroutines.
#
#               Known limitations:
#                   1. None
#
# Date       Author/Email                Description
# ---------- --------------------------- ---------------------------------------
# 03/18/2003 Jim Lancaster               Version 0.1
#            jlancaster@mbdworld.com
#
# Known issues:
#   1. None
#
# Future plans:
#   1. Add in ifEntry stats/graphs.
#   2. Use ifDescr to provide port name.
#   4. Sort the ports by port number in DS,graphs & graph hierarchy.
#
#===============================================================================
use strict;

# Libraries of common subroutines. Start by looking in the current directory.
if (open(SUBS,"subs_misc.pl")) {
    require "subs_misc.pl";
    require "subs_snmp.pl";
    require "subs_cacti.pl";
# If this script is being called from within cacti, they will be found here.
} elsif (open(SUBS,"scripts/subs_misc.pl")) {
    require "scripts/subs_misc.pl";
    require "scripts/subs_snmp.pl";
    require "scripts/subs_cacti.pl";
} else {
    die "\n\nUnable to open subroutine libraries!";
}

# Used for command line parameters.
use Getopt::Long;

# Used to search for devices that respond to icmp ping.
use Net::Ping;
use vars qw($p);

# Used to test if SNMP port is open.
use IO::Socket;

# Used to gather SNMP information from SNMP-enabled devices.
use Net::SNMP;
use vars qw($session $error $response);

# Used for writing results to cacti database.
use DBI;

#===============================================================================
#           Constants
#===============================================================================
my $TRUE  = 1;
my $FALSE = 0;
my $YES = $TRUE;
my $NO = $FALSE;
my $SUCCESS = $TRUE;
my $FAILURE = $FALSE;
my $PROCESS_ID = $$;
my $SCRIPT_NAME = $0;
my $MAX_WAIT_TIME = 300;
my $WAIT_TIME = 10;
my $AUTHOR = "Jim Lancaster";
my $EMAIL = "jlancaster\@mbdworld.com";
my $COPYRIGHT = "Copyright 2003";
my $TITLE = "RMON Switch Performance Report";
my $VERSION = "Version 0.1";
my $SCRIPT_OWNER = "MBD Network Services, LLC";

#===============================================================================
#           Script Parameters
#===============================================================================
my $usageText = "
$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER
$AUTHOR \($EMAIL\)


Usage: [perl] $SCRIPT_NAME -i=ipaddress [-q -h -? -c]

  -i=ipaddress IP address of host                       (required)
  -n=number    Interface number                         (optional)
  -q           Return data only--no headers or footers  (optional)
  -h,-?        Display usage text                       (optional)
  -c           Create the cacti DIS, DS & graphs        (optional)

$SCRIPT_NAME retrieves RMON performance information using SNMP.

With the '-n' and '-q' parameters, the script returns the performance data
in a single line format suitable for use with Cacti. Note that both
parameters must be present.

With the '-c' parameter, this script will create the DIS (if necessary),
the DS, and a basic set of graphs FOR ALL PORTS. This should allow you to
begin graphing data within minutes of successfully installing cacti. The
graphs can be modified to suit your needs using cacti's built-in tools at
any time. If the '-n' parameter is specified with '-c', it will be ignored.

Your feedback is welcome. This script should be flexible enough to be
stretched in a number of ways. All I need is your input to guide me.";

#===============================================================================
#           Reports Array
#===============================================================================

#===============================================================================
#           Global Variables
#===============================================================================
# From Main
my (%hoh_cpu,%hoh_mem,%hoh_disk,%hoh_if,%hoh_prn,%hoh_rmon);
my (@output,@outputStr);
my (%hoh_if2,%hoh_rmon2);

#===============================================================================
#           Main Begin
#===============================================================================
# Read command line options.
my %parm = &cmdLineOpts();

# Echo basic script information to the screen during runtime.
if (!$parm{'quiet'}) {
    print STDOUT "$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER\n";
    print STDOUT "$AUTHOR \($EMAIL\)\n\n";
}

# Go to usage if the command line input specifies it.
if ($parm{'help'}) {
    &usage($usageText)
};

# Go to usage if no IP address is specified.
if (!$parm{'ipaddress'}) {
    &usage($usageText)
};

# If the read community string and port are not specified at the
#  command line, set them to the default values.
if (!$parm{'rcs'}) {$parm{'rcs'}="public"};
if (!$parm{'port'}) {$parm{'port'}="161"};

# Ping the device using icmp.
$p = Net::Ping->new('icmp');
if ($p->ping($parm{'ipaddress'},1)) {  # Timeout = 1 second.

    # Get the SNMP performance data.
    my $hostname = &snmpStats();

    if ($hostname) {

        # Configure Cacti to poll and graph this host if the command line input specifies it.
        if ($parm{'create'}) {

            # Get the cacti MySQL database settings;
            my %cactiDB     = &cactiDB();
            my ($dsn)       = "DBI:mysql:$cactiDB{'default'}:$cactiDB{'hostname'}";
            my ($username)  = "$cactiDB{'username'}";
            my ($password)  = "$cactiDB{'password'}";

            # Connect to the cacti MySQL database.
            my $dbh = DBI->connect($dsn,$username,$password)
                or die "Can't connect to MySQL database: $DBI::errstr\n";

            # Check the discover_hosts table to see if the hostname needs to be overridden.
            my $query = "SELECT name FROM cacti.discover_hosts WHERE ipaddress = '$parm{'ipaddress'}'";
            my $newHostname = $dbh->selectrow_array ($query);
            if ($newHostname) {$hostname = $newHostname;}

            # Note: Must use %hoh_if key because not all interfaces have RMON stats (e.g.,VLANs)
            foreach my $key (keys %hoh_if) {

                my $DISName;

                # Create a unique %hoh_if & %hoh_rmon for each port. We do this because
                #  the subs in subs_cacti expect hoh and not just hashes. This allows us
                #  to use the same subs for creating cacti entries for servers as well
                #  as individual switch ports. (Servers may have more than one interface.)
                %hoh_if2 = ();
                %hoh_rmon2 = ();
                $hoh_if2{'if'} = {%{$hoh_if{$key}}};
                if (defined(%{$hoh_rmon{$key}})) {
                    $hoh_rmon2{'rmon'} = {%{$hoh_rmon{$key}}};

                    # Set the DIS name.
                    $DISName = "Get Network Traffic via SNMP (if/rmon)";
                } else {
                    $DISName = "Get Network Traffic via SNMP (if)";
                }

                # Create a descriptive device name.
                my $devicename = $hostname."-".${$hoh_if{$key}}{'ifDescr'};

                # For testing purposes only.
                print "Port name => ".$devicename."\n";
                &dref($hoh_if2{'if'},"hash");
                &dref($hoh_rmon2{'rmon'},"hash");
                print "\n";

                # Initialize arrays and build output,output string.
                @output     = ();
                @outputStr  = ();
                &buildOutput($hoh_if2{'if'},$hoh_rmon2{'rmon'});

                # Create the data input source (DIS)
                my ($newSrcID,$inFieldID,$outFieldID)  = &createDIS($dbh,$DISName,$parm{'ipaddress'},'switch.pl -i=<ipaddress> -n=<index> -q',\@outputStr);

                # Create the srcInput hash. There should be one for each input field specified
                #  in '<>' in the preceding command.
                my %srcInput;
                $srcInput{'ipaddress'}  = $parm{'ipaddress'};
                $srcInput{'index'}      = $key;

                # In order to keep track of the type of hash that is referenced in srcData array,
                #  each array element number needs to be predetermined. Unused elements can be
                #  commented-out.
                my @srcData;
                #$srcData[0]     = \%hoh_cpu;
                #$srcData[1]     = \%hoh_mem;
                #$srcData[2]     = \%hoh_disk;
                $srcData[3]     = \%hoh_if2;
                #$srcData[4]     = \%hoh_prn;
                $srcData[5]     = \%hoh_rmon2;

                # Create the data source (DS)
                my ($ds,$newDSID) = &createDS($dbh,$devicename,$newSrcID,$inFieldID,$outFieldID,\%srcInput,\@srcData);

                # Create the graphs
                my ($rrdGraphID) = &createGraph($dbh,$devicename,$newDSID,\@srcData);

                # Add the graphs to the graph hierarchy.
                my %hierarchy;
                $hierarchy{'root'}       = "Auto-created";
                $hierarchy{'heading'}    = "Internetworking";
                $hierarchy{'hostname'}   = $hostname;                  # Optional on servers.
                $hierarchy{'device'}     = $devicename;
                &createGraphHierarchyTree($dbh,$rrdGraphID,\%hierarchy);

            }

            # Terminate the database connection.
            $dbh->disconnect ()
                or die "Disconnection failed: $DBI::errstr\n";
        }

        if ($parm{'number'}) {

            # Echo report completion to the screen, or print data in Cacti acceptable format.
            if (!$parm{'quiet'}) {

                # Create a descriptive device name.
                my $devicename = $hostname."-".${$hoh_if{$parm{'number'}}}{'ifDescr'};

                print "Print results for $parm{'ipaddress'}:\n\n";
                print "Device => $devicename\n";
                #&dref(\%hoh_cpu,'hoh');
                #&dref(\%hoh_mem,'hoh');
                #&dref(\%hoh_disk,'hoh');
                &dref(\%hoh_if,'hoh');
                #&dref(\%hoh_prn,'hoh');
                &dref(\%hoh_rmon,'hoh');
                print STDOUT "\nResults for $parm{'ipaddress'} complete.\n\n"

            } else {

                # Build output & output string.
                @output     = ();
                @outputStr  = ();

                # For testing purposes only.
                #dref($hoh_if{$parm{'number'}},"hash");
                #dref($hoh_rmon{$parm{'number'}},"hash");

                &buildOutput($hoh_if{$parm{'number'}},$hoh_rmon{$parm{'number'}});

                # Print output in Cacti acceptable format:
                print join(" ",@output);
            }
        }
    }

} else {

    print "$parm{'ipaddress'} - did not respond.\n";

}

#===============================================================================
#           Main End
#===============================================================================

#===============================================================================
#           Subroutines
#===============================================================================
# cmdLineOpts()
#
# Description - Check command line options
#
# Inputs:
#  1. None
#
# Processing:
#  1. Read the command line options and store the results in a hash.
#
# Outputs:
#  1. %parm - A hash containing the command line options.
#
#===============================================================================
sub cmdLineOpts
{
    # Check for command line options. Store them in %parm hash if present.
    my %parm;
    Getopt::Long::Configure("prefix_pattern=(-|--|\/)");
    GetOptions(\%parm, qw(
        help|?|h
        ipaddress|i=s
        number|n=s
        rcs|r=s
        port|p=s
        create|c
        quiet|q
        ));

    return(%parm);
}

#===============================================================================
# snmpStats()
#
# Descripton - Use SNMP to gather the performance data.
#
# Inputs:
#  1.
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#===============================================================================
sub snmpStats
{

    my %sysValues;

    # Check to see if SNMP port is responding
    my $remote = IO::Socket::INET -> new(
            Proto      => "udp",
            PeerAddr   => $parm{'ipaddress'},
            PeerPort   => $parm{'port'},
            );

    # If the SNMP port is open, attempt to use SNMP to gather
    #  more information about the device.
    if ($remote) {
        close $remote;

        # Attempt to create an SNMP session.
        ($session, $error) = Net::SNMP->session(
            -hostname    => $parm{'ipaddress'},
            -community   => $parm{'rcs'},
            -port        => $parm{'port'},
            -timeout     => 1
            );

        # Retrieve a hash of system OID values
        my ($systemBaseOID,$systemOIDs) = &mib2System();
        %sysValues = &snmpValue($parm{'ipaddress'},$session,"0",$systemOIDs);

        # Fix those conditions where sysName has not been set.
        if ($sysValues{'sysDescr'} & $sysValues{'sysName'} eq "") {
            $sysValues{'sysName'} = "<blank>";
        }

        # Retrieve hash of hashes of ifEntry OID values
        my ($ifEntryBaseOID,$ifEntryOIDs) = &mib2If();
        my ($etherStatsEntryBaseOID,$etherStatsEntryOIDs) = &mib2EtherStats();
        if ($parm{'number'}) {
            my %ifValues                = &snmpValue($parm{'ipaddress'},$session,$parm{'number'},$ifEntryOIDs);
            $hoh_if{$parm{'number'}}    = {%ifValues};

            my %rmonValues              = &snmpValue($parm{'ipaddress'},$session,$parm{'number'},$etherStatsEntryOIDs);
            if (%rmonValues) {
                $hoh_rmon{$parm{'number'}}  = {%rmonValues};
            }
        } else {
            my %hoh_ifValues    = &snmpTable($parm{'ipaddress'},$session,$$ifEntryOIDs{'ifIndex'},$ifEntryOIDs);
            my %hoh_rmonValues  = &snmpTable($parm{'ipaddress'},$session,$$etherStatsEntryOIDs{'rmonIndex'},$etherStatsEntryOIDs);

            # Retain only the interesting port values.
            foreach my $key1 (keys %hoh_ifValues) {

                # We are only interested in port types 6 (ethernet) and 53 (VLAN).
                #  I've also noticed that the empty transceiver ports (25 & 26) show up on
                #  my HP2524 switch with no description. I'm guessing that the empty ifDescr
                #  gets filled in when the transceivers are installed. I am masking out
                #  the ports with empty ifDescr fields until I recieve more information.
                if (${$hoh_ifValues{$key1}}{'ifType'} =~ /(6|53)/ & ${$hoh_ifValues{$key1}}{'ifDescr'} ne "") {
                    $hoh_if{$key1}      = \%{$hoh_ifValues{$key1}};

                    if (defined(%{$hoh_rmonValues{$key1}})) {
                        $hoh_rmon{$key1} = \%{$hoh_rmonValues{$key1}};
                    }
                }
            }
        }

        $session->close;
    }

    # For testing purposes only.
    #&dref(\%hoh_if,"hoh");
    #&dref(\%hoh_rmon,"hoh");
    #print values(%hoh_if).",".values(%hoh_rmon)."\n";

    return($sysValues{'sysName'});
}

#===============================================================================
# buildOutput()
#
# Description - Build output & outputStr for Cacti.
#
# Inputs:
#  1.
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#===============================================================================
sub buildOutput
{

    my ($if,$rmon) = @_;


    # Interface stats
    if ($if) {
        push @output,$$if{'ifInOctets'};
        push @output,$$if{'ifInErrors'};
        push @output,$$if{'ifInDiscards'};
        push @output,$$if{'ifOutOctets'};
        push @output,$$if{'ifOutErrors'};
        push @output,$$if{'ifOutDiscards'};
        push @outputStr,"ifInOctets";
        push @outputStr,"ifInErrors";
        push @outputStr,"ifInDiscards";
        push @outputStr,"ifOutOctets";
        push @outputStr,"ifOutErrors";
        push @outputStr,"ifOutDiscards";
    }

    # RMON stats.
    if ($rmon) {
        push @output,$$rmon{'rmonDropEvents'};
        push @output,$$rmon{'rmonOctets'};
        push @output,$$rmon{'rmonPkts'};
        push @output,$$rmon{'rmonBroadcastPkts'};
        push @output,$$rmon{'rmonMulticastPkts'};
        push @output,$$rmon{'rmonCRCAlignErrors'};
        push @output,$$rmon{'rmonUndersizePkts'};
        push @output,$$rmon{'rmonOversizePkts'};
        push @output,$$rmon{'rmonFragments'};
        push @output,$$rmon{'rmonJabbers'};
        push @output,$$rmon{'rmonCollisions'};
        push @output,$$rmon{'rmonPkts64Octets'};
        push @output,$$rmon{'rmonPkts65to127Octe'};
        push @output,$$rmon{'rmonPkts128to255Oct'};
        push @output,$$rmon{'rmonPkts256to511Oct'};
        push @output,$$rmon{'rmonPkts512to1023Oc'};
        push @output,$$rmon{'rmonPkts1024to1518O'};
        push @outputStr,"rmonDropEvents";
        push @outputStr,"rmonOctets";
        push @outputStr,"rmonPkts";
        push @outputStr,"rmonBroadcastPkts";
        push @outputStr,"rmonMulticastPkts";
        push @outputStr,"rmonCRCAlignErrors";
        push @outputStr,"rmonUndersizePkts";
        push @outputStr,"rmonOversizePkts";
        push @outputStr,"rmonFragments";
        push @outputStr,"rmonJabbers";
        push @outputStr,"rmonCollisions";
        push @outputStr,"rmonPkts64Octets";
        push @outputStr,"rmonPkts65to127Octe";
        push @outputStr,"rmonPkts128to255Oct";
        push @outputStr,"rmonPkts256to511Oct";
        push @outputStr,"rmonPkts512to1023Oc";
        push @outputStr,"rmonPkts1024to1518O";
    }

}

#===============================================================================
