#===============================================================================
# Program:      w2kperf.pl
#
# Description:  This script gathers basic performance information via WMI for
#               use with cacti v0.6.8a.  It is also capable of creating the DIS,
#               DS entries in cacti for the polled object, as well as a set of
#               basic graphs.
#
#               This script was developed under ActiveState Perl version 5.6.1.
#
# Requirements: This ActiveState PERL must be installed on the system where the
#               script is executed.
#
#               The following additional Perl modules must also be installed:
#                   Win32::OLE - for all WMI calls
#                   DBI - for interfacing with MySQL
#                   DBD:MySQL - for interfacing with MySQL
#
#               The following files must exist in the same directory as this
#               script:
#                   subs_misc.pl - A library of miscellaneous subroutines.
#                   subs_snmp.pl - A library of SNMP subroutines.
#                   subs_cacti.pl - A library of cacti subroutines.
#
#               Known limitations:
#                   1. Windows 2000/XP - none.
#                   2. Windows NT4 - Requires WMINT4.EXE (WBEMCORE v1.5) or
#                       later.
#                   3. Windows 9x/ME - not tested. (Why bother?)
#                   4. Non-Windows systems (Linux) - This script will NOT run.
#
#
# Date       Author/Email                Description
# ---------- --------------------------- ---------------------------------------
# 02/17/2003 Jim Lancaster               Version 0.1
#            jlancaster@mbdworld.com
#
# 02/19/2003 Jim Lancaster               Version 0.1a
#            jlancaster@mbdworld.com     Added command line parameter (-c) to
#                                         create a DIS in cacti.
#
# 02/21/2003 Jim Lancaster               Version 0.2
#            jlancaster@mbdworld.com     Pull computername from WMI for use in
#                                        creating a DS in cacti.
#                                        Completely rewrote the polling script
#                                          to store results in srcData hash to
#                                         simplify creation of DIS/DS.
#                                        Still have unresolved problem with
#                                         creation of DIS.
#
# 02/21/2003 Jim Lancaster               Version 0.2a
#            jlancaster@mbdworld.com     Fixed problem with DIS;
#
# 02/23/2003 Jim Lancaster               Version 0.3
#            jlancaster@mbdworld.com     Added creation of standard graphs.
#                                        Updated usage text and script header.
#                                        Added check to see if DIS already
#                                         exists; reuse existing DIS if it does.
#                                        Added creation of graph & graph items.
#                                        Script submitted to Ian Berry (raX) &
#                                         Piotr Bulczak (bulek) for testing.
#
# 02/23/2003 Jim Lancaster               Version 0.3a
#            jlancaster@mbdworld.com     Added creation of graph hierarchy.
#                                        Modified w2kperf.cfg to include new
#                                         options for DSPrefix name &
#                                         GraphHierarchy.
#                                        Script submitted to Ian Berry (raX) &
#                                         Piotr Bulczak (bulek) for testing.
#
# 03/03/2003 Jim Lancaster               Version 0.4 - Begin complete rewrite
#            jlancaster@mbdworld.com     Created new function to obtain db
#                                         settings from cacti config.php.
#                                        Modified require statement to locate
#                                         miscsubs.pl when this script is called
#                                         from within cacti.
#                                        Added wmi/snmp if stats.
#                                        Combined graphs into a standard set.
#
# 03/18/2003 Jim Lancaster               Version 0.4a - Complete rewrite finished.
#            jlancaster@mbdworld.com     Dropped the w2kperf.cfg idea.
#                                        Added ping test to make sure host is up.
#                                        Incorporated modifications stemming from
#                                         SNMP polling code.
#
#
# Known issues:
#   1. Apparent bug in cacti 0.6.8a - If you edit one of the graph items and
#       save it, the rrd filename referenced by that item gets changed to that
#       of the ds (or a truncated version thereof).  This causes the graph to
#       disappear. I think the problem lies in the graph_items.php code, but
#       further research is needed. Reported this to Ian on 2/21/03.
#   2. There appears to be some kind of wierd name-sorting issue in displaying
#       the current data sources in cacti. Most of the mult-source DS's created
#       by this script show up correctly, however the 'BLACKBOX' line items show
#       up beneath 'BLACKBOX-BETA' items in testing. Nevertheless, both DS
#       appear to be functioning correctly.
#   3. 3/12/03 - WMI calls appear to hang indefinitely on some boxes. Need to
#       research how to set a timeout value.
#
# Future plans:
#   1. Add "rollback" mechanism to reverse all of the MySQL record entries made
#       by the '-c' option.
#
#===============================================================================
use strict;

# Libraries of common subroutines. Start by looking in the current directory.
if (open(SUBS,"subs_misc.pl")) {
    require "subs_misc.pl";
    require "subs_snmp.pl";
    require "subs_cacti.pl";
# If this script is being called from within cacti, they will be found here.
} elsif (open(SUBS,"scripts/subs_misc.pl")) {
    require "scripts/subs_misc.pl";
    require "scripts/subs_snmp.pl";
    require "scripts/subs_cacti.pl";
} else {
    die "\n\nUnable to open subroutine libraries!";
}

# Used for command line parameters.
use Getopt::Long;

# Used to test whether specified host is active.
use Net::Ping;
use vars qw($p);

# Used to gather WMI information from Windows devices.
use Win32::OLE qw(in);

# Required for writing to MySQL database.
use DBI;

#===============================================================================
#                   Constants
#===============================================================================
my $TRUE  = 1;
my $FALSE = 0;
my $YES = $TRUE;
my $NO = $FALSE;
my $SUCCESS = $TRUE;
my $FAILURE = $FALSE;
my $PROCESS_ID = $$;
my $SCRIPT_NAME = $0;
my $MAX_WAIT_TIME = 300;
my $WAIT_TIME = 10;
my $AUTHOR = "Jim Lancaster";
my $EMAIL = "jlancaster\@mbdworld.com";
my $COPYRIGHT = "Copyright 2003";
my $TITLE = "MS Windows 2000 Performance Report";
my $VERSION = "Version 0.4a";
my $SCRIPT_OWNER = "MBD Network Services, LLC";

#===============================================================================
#                   Script Parameters
#===============================================================================
my $usageText = "
$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER
$AUTHOR \($EMAIL\)


Usage: [perl] $SCRIPT_NAME -i=ipaddress [-q -h -? -c]

  -i=ipaddress IP address or host name of device        (required)

  -q           Return data only--no headers or footers  (optional)
  -h,-?        Display usage text                       (optional)
  -c           Create the cacti DIS, DS & graphs        (optional)

$SCRIPT_NAME retrieves Windows NT4/2000 server performance information
using WMI. Without the '-q' parameter, the script prints the the gathered
information to the screen. Use this for testing purposes to see if the
remote device is accessible and to see the number of host attributes
the script is returning performance data for.

With the '-q' parameter, the script returns the performance data in a
single line format suitable for use with Cacti.

With the '-c' parameter, this script will create the DIS (if necessary),
the DS, and a basic set of graphs. This should allow you to begin
graphing data within minutes of successfully installing cacti. The graphs
can be modified to suit your needs using cacti's built-in tools at any
time thereafter.

Because this script will return performance data for all CPUs, disks, and
interfaces, the returned results will vary from server to server and will
change if new devices are added to a server that has already been setup.
This WILL cause a problem if the number of outputs defined in the cacti DIS
do not agree with the output returned by this script. For this reason, it
is STRONGLY recommended that you use the '-c' option to create the DIS, DS
and graph records for you.

Your feedback is welcome. This script should be flexible enough to be
stretched in a number of ways. All I need is your input to guide me.";

#===============================================================================
#                   Reports Array
#===============================================================================

#===============================================================================
#                   Global Variables
#===============================================================================
# From Main
my ($hostname,%hoh_cpu,%hoh_mem,%hoh_disk,%hoh_if);
my (@output,@outputStr);

#===============================================================================
#                   Main Begin
#===============================================================================
# Read command line options.
my %parm = &cmdLineOpts();

# Echo basic script information to the screen during runtime.
if (!$parm{'quiet'}) {
    print STDOUT "$TITLE - $VERSION\n$COPYRIGHT - $SCRIPT_OWNER\n";
    print STDOUT "$AUTHOR \($EMAIL\)\n\n";
}

# Go to usage if the command line input specifies it.
if ($parm{'help'}) {
    &usage($usageText)
};

# Go to usage if no IP address or computername is specified.
if (!$parm{'ipaddress'}) {
    &usage($usageText)
};

# Ping the device using icmp.
$p = Net::Ping->new('icmp');
if ($p->ping($parm{'ipaddress'},1)) {  #Timeout = 1 second.

    # Get the WMI stats.
    $hostname = &wmiStats();

    if ($hostname) {

        # Display contents of hashes.
        if (!$parm{'quiet'}) {
            print "Print results for $parm{'ipaddress'}:\n\n";
            print "Computername => $hostname\n";
            &dref(\%hoh_cpu,'hoh');
            &dref(\%hoh_mem,'hoh');
            &dref(\%hoh_disk,'hoh');
            &dref(\%hoh_if,'hoh');
        }

        # Build output & output string.
        &buildOutput();

        # Echo report completion to the screen, or print data in Cacti acceptable format.
        if (!$parm{'quiet'}) {
            print STDOUT "\nResults for $parm{'ipaddress'} complete.\n\n"
        } else {
            # Print output in Cacti acceptable format:
            print join(" ",@output);
        }

        # Configure Cacti to poll and graph this host if the command line input specifies it.
        if ($parm{'create'}) {

            # Get the cacti MySQL database settings;
            my %cactiDB     = &cactiDB();
            my ($dsn)       = "DBI:mysql:$cactiDB{'default'}:$cactiDB{'hostname'}";
            my ($username)  = "$cactiDB{'username'}";
            my ($password)  = "$cactiDB{'password'}";

            # Connect to the cacti MySQL database.
            my $dbh = DBI->connect($dsn,$username,$password)
                or die "Can't connect to MySQL database: $DBI::errstr\n";

            my $countCPU    = values(%hoh_cpu);
            my $countDisk   = values(%hoh_disk);
            my $countIf     = values(%hoh_if);
            my $DISName     = "Get Windows Performance via WMI (C-$countCPU/D-$countDisk/I-$countIf)";

            # Create the data input source (DIS)
            my ($newSrcID,$inFieldID,$outFieldID)  = &createDIS($dbh,$DISName,$parm{'ipaddress'},'w2kperf.pl -i=<ipaddress> -q',\@outputStr);

            # Create the srcInput hash. There should be one for each input field specified
            #  in '<>' in the preceding command.
            my %srcInput;
            $srcInput{'ipaddress'}  = $parm{'ipaddress'};

            # In order to keep track of the type of hash that is referenced in srcData array,
            #  each array element number needs to be predetermined. Unused elements can be
            #  commented-out.
            my @srcData;
            $srcData[0]     = \%hoh_cpu;
            $srcData[1]     = \%hoh_mem;
            $srcData[2]     = \%hoh_disk;
            $srcData[3]     = \%hoh_if;
            #$srcData[4]     = \%hoh_prn;

            # Create the data source (DS)
            my ($ds,$newDSID)   = &createDS($dbh,$hostname,$newSrcID,$inFieldID,$outFieldID,\%srcInput,\@srcData);

            # Create the graphs
            my ($rrdGraphID)    = &createGraph($dbh,$hostname,$newDSID,\@srcData);

            # Add the graphs to the graph hierarchy.
            my %hierarchy;
            $hierarchy{'root'}       = "Auto-created";
            $hierarchy{'heading'}    = "Windows Servers";
            #$hierarchy{'hostname'}   = $hostname;                  # Optional.
            $hierarchy{'device'}     = $hostname;
            &createGraphHierarchyTree($dbh,$rrdGraphID,\%hierarchy);

            # Terminate the database connection.
            $dbh->disconnect ()
                or die "Disconnection failed: $DBI::errstr\n";
        }

    }

} else {

    print "$parm{'ipaddress'} - did not respond.\n";

}

#===============================================================================
#                   Main End
#===============================================================================

#===============================================================================
#                   Subroutines
#===============================================================================
# cmdLineOpts()
#
# Description - Check command line options
#
# Inputs:
#  1. None
#
# Processing:
#  1. Read the command line options and store the results in a hash.
#
# Outputs:
#  1. %parm - A hash containing the command line options.
#
#===============================================================================
sub cmdLineOpts
{
    # Check for command line options. Store them in %parm hash if present.
    my %parm;
    Getopt::Long::Configure("prefix_pattern=(-|--\\/)");
    GetOptions(\%parm, qw(
                help|?|h
                ipaddress|i=s
                create|c
                quiet|q
                ));

    return(%parm);
}

#===============================================================================
# wmiStats()
#
# Description - Get the WMI performance stats.
#
# Inputs:
#  1.
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#===============================================================================
sub wmiStats
{

    my ($wmi,$services,$count);

    # WMI must be accessible locally in order to make WMI calls to remote Windows hosts.
    $wmi = Win32::OLE->new('WbemScripting.SWbemLocator') ||
         die "Cannot access WMI on local machine!\n\n", Win32::OLE->LastError;

    # Attempt to connect to the cimv2 namespace on the remote host.
    $services = $wmi->ConnectServer($parm{'ipaddress'},"root/cimv2")
        or die "Cannot access WMI on remote machine: $parm{'ipaddress'}!\n\n", Win32::OLE->LastError;

    if (defined($services)) {

        # Start by getting the computer name.
        foreach my $dev (in($services->InstancesOf("Win32_ComputerSystem"))) {
            $hostname                   = $dev->{'Name'};
        }

        # Next, get the load on each CPU and store in a hash of hashes.
        my %cpu;
        $count = 0;
        foreach my $dev (in($services->InstancesOf("Win32_Processor"))) {
            $cpu{'PercentCPULoad'}          = $dev->{'LoadPercentage'};
            $cpu{'Descr'}                   = $dev->{'DeviceID'};
            $hoh_cpu{$count}                = {%cpu};
            $count++;
        }

        # For testing purposes only.
        #&dref(\%hoh_cpu,"hoh");

        # Next, get the available memory and store in a hash of hashes. Start by getting
        #  the total memory; normalize the data to bytes.
        my %mem;
        foreach my $dev (in($services->InstancesOf("Win32_LogicalMemoryConfiguration"))) {
            $mem{'TotalPhysicalMemory'}     = $dev->{'TotalPhysicalMemory'} * 1024;
            $mem{'TotalPageFileSpace'}      = $dev->{'TotalPageFileSpace'} * 1024;
            $mem{'TotalVirtualMemory'}      = $dev->{'TotalVirtualMemory'} * 1024;
        }

        # Now, get the free memory; normalize the data to bytes.
        foreach my $dev (in($services->InstancesOf("Win32_OperatingSystem"))) {
            $mem{'BytesFreeMem'}            = $dev->{'FreePhysicalMemory'} * 1024;
            $mem{'BytesFreePageFile'}       = $dev->{'FreeSpaceInPagingFiles'} * 1024;
            $mem{'BytesFreeVirtMem'}        = $dev->{'FreeVirtualMemory'} * 1024;
        }

        # Now, calculate free memory percentages.
        $mem{'PercentFreeMem'}              = sprintf("%.0f",($mem{'BytesFreeMem'} /
                                                    $mem{'TotalPhysicalMemory'}) * 100);
        $mem{'PercentFreePageFile'}         = sprintf("%.0f",($mem{'BytesFreePageFile'} /
                                                    $mem{'TotalPageFileSpace'}) * 100);
        $mem{'PercentFreeVirtMem'}          = sprintf("%.0f",($mem{'BytesFreeVirtMem'} /
                                                    $mem{'TotalVirtualMemory'}) * 100);

        # Finally, store the results in a hash of hashes.
        $hoh_mem{'Memory'}                  = {%mem};

        # For testing purposes only.
        #&dref(\%hoh_mem,"hoh");

        # Next, get the free disk space on each local logical disk and store in a hash of hashes.
        my %disk;
        $count = 0;
        foreach my $dev (in($services->InstancesOf("Win32_LogicalDisk"))) {

            # Confine the report to local hard disks (DriveType 3).
            if ($dev->{'DriveType'} == 3) {

                # Store the results in a hash of hashes.
                $disk{'BytesFreeDisk'}      = $dev->{'FreeSpace'};
                $disk{'Size'}               = $dev->{'Size'};
                $disk{'PercentFreeDisk'}    = sprintf("%.0f",($dev->{'FreeSpace'} / $dev->{'Size'}) * 100);
                $disk{'Descr'}              = $dev->{'DeviceID'};
                $hoh_disk{$count} = {%disk};
                $count++;

            }
        }
    }

    # Attempt to connect to the snmp namespace on the remote host.
    $services = $wmi->ConnectServer($parm{'ipaddress'}, "root/snmp/localhost");
    if (defined($services)) {

        # Create an array of WMI interface 'instances' we are interested in.
        my @instances = ("ifInOctets","ifInErrors","ifInDiscards",
                            "ifOutOctets","ifOutErrors","ifOutDiscards");

        my %if;
        $count = 0;
        foreach my $dev (in($services->InstancesOf("SNMP_RFC1213_MIB_ifTable"))) {

            # Skip the software loopback interface and any interface that is not in use.
            if ($dev->{'ifType'} ne "softwareLoopback" & $dev->{'ifAdminStatus'} eq "up") {
                foreach my $inst (in @instances) {

                    # Store the results in a hash.
                    $if{$inst}              = $dev->{$inst};

                }
                $if{'ifDescr'}              = $dev->{'ifDescr'};

                # Store the results in a hash of hashes.
                $hoh_if{$count}  = {%if};
                $count++;
            }
        }

        # For testing purposes only.
        #&dref(\%hoh_if,"hoh");

    }

    return ($hostname);

}

#===============================================================================
# buildOutput()
#
# Description - Build output & outputStr for Cacti.
#
# Inputs:
#  1.
#
# Processing:
#  1.
#
# Outputs:
#  1.
#
#===============================================================================
sub buildOutput
{

    # CPU
    foreach my $key (sort keys %hoh_cpu) {
        push @output,$hoh_cpu{$key}{'PercentCPULoad'};
        push @outputStr,"PercentCPULoad-".$key;
    }

    # Memory
    push @output,$hoh_mem{'Memory'}{'BytesFreeMem'};
    push @output,$hoh_mem{'Memory'}{'BytesFreePageFile'};
    push @output,$hoh_mem{'Memory'}{'BytesFreeVirtMem'};
    push @output,$hoh_mem{'Memory'}{'PercentFreeMem'};
    push @output,$hoh_mem{'Memory'}{'PercentFreePageFile'};
    push @output,$hoh_mem{'Memory'}{'PercentFreeVirtMem'};
    push @outputStr,"BytesFreeMem";
    push @outputStr,"BytesFreePageFile";
    push @outputStr,"BytesFreeVirtMem";
    push @outputStr,"PercentFreeMem";
    push @outputStr,"PercentFreePageFile";
    push @outputStr,"PercentFreeVirtMem";

    # Disk
    foreach my $key (sort keys %hoh_disk) {
        push @output,$hoh_disk{$key}{'BytesFreeDisk'};
        push @output,$hoh_disk{$key}{'PercentFreeDisk'};
        push @outputStr,"BytesFreeDisk-".$key;
        push @outputStr,"PercentFreeDisk-".$key;
    }

    # Interface
    foreach my $key (sort keys %hoh_if) {
        push @output,$hoh_if{$key}{'ifInOctets'};
        push @output,$hoh_if{$key}{'ifInErrors'};
        push @output,$hoh_if{$key}{'ifInDiscards'};
        push @output,$hoh_if{$key}{'ifOutOctets'};
        push @output,$hoh_if{$key}{'ifOutErrors'};
        push @output,$hoh_if{$key}{'ifOutDiscards'};
        push @outputStr,"ifInOctets-".$key;
        push @outputStr,"ifInErrors-".$key;
        push @outputStr,"ifInDiscards-".$key;
        push @outputStr,"ifOutOctets-".$key;
        push @outputStr,"ifOutErrors-".$key;
        push @outputStr,"ifOutDiscards-".$key;
    }

    # For testing purposes only.
    #print join("\n",@outputStr)."\n";
    #print join(",",@output)."\n";

}

#===============================================================================
